// -*- c++ -*-
//
// Copyright 2010 Ovidiu Predescu <ovidiu@gmail.com>
// Date: June 2010
// Updated: 08-JAN-2012 for Arduno IDE 1.0 by <Hardcore@hardcoreforensics.com>
// Extended July 2012 by Ontje Helmich <hello@ontjehelmich.de> to fit project needs

#include <SPI.h>
#include <Ethernet.h>
#include <Flash.h>
#include <SD.h>
#include <TinyWebServer.h>

boolean file_handler(TinyWebServer& web_server);
boolean index_handler(TinyWebServer& web_server);

boolean has_filesystem = true;

boolean status, firstLed; //Status der Herd-LED, bei Programmstart AN

Sd2Card card;
SdVolume volume;
SdFile root;
SdFile file;

int sensorMagnetTuerWert = 0; // Wert für den Magnetsensor der Tür
int sensorMagnetFensterWert = 0;  // Wert für den Magnetsensor des Fensters
int sensorLichtWert = 0; // Wert für den Lichtsensor im Kühlschrank
long sensorMagnetTuerZuletztGelesen = 0; // Wert für das zeitliche Auslesen des Sensorwerts der Tür
long sensorMagnetFensterZuletztGelesen = 0; // Wert für das zeitliche Auslesen des Sensorwerts des Fensters
long sensorLichtZuletztGelesen = 0; // Wert für das zeitliche Auslesen des Sensorwerts des Kühlschranks

static uint8_t mac[] = { 0x90, 0xA2, 0xDA, 0x00, 0x82, 0xCC }; // MAC-Adresse des Ethernet-Shields

byte ip[] = { 192, 168, 2, 8 }; // IP-Adresse des Arduinos im Netzwerk

const int sdChipSelect = 4; // SD card chipSelect

TinyWebServer::PathHandler handlers[] = { // Pathhandler gibt an, wo die einzelnen Sensorwerte abgespeichert werden
  {"/", TinyWebServer::GET, &index_handler },
  {"/sensor/MagnetFenster", TinyWebServer::GET, &sensor_MagnetFenster_handler },
  {"/sensor/Licht", TinyWebServer::GET, &sensor_Licht_handler },
  {"/sensor/Herd", TinyWebServer::GET, &sensor_Herd_handler },
  {"/" "*", TinyWebServer::GET, &file_handler },
  {NULL},
};


boolean file_handler(TinyWebServer& web_server) {
  char* filename = TinyWebServer::get_file_from_path(web_server.get_path());
  send_file_name(web_server, filename);
  free(filename);
  return true;
}


// diese Funktion wird vom loop() aufgerufen und speichert den aktuellen Sensorwert
void sensorMagnetTuerAuslesen() {
  sensorMagnetTuerWert = digitalRead(2);}
void sensorMagnetFensterAuslesen() {
  sensorMagnetFensterWert = digitalRead(3);}
void sensorLichtAuslesen() {
  sensorLichtWert = analogRead(4);
}

// Sensorhandler für Fenster
boolean sensor_MagnetFenster_handler(TinyWebServer& web_server) {
  web_server.send_error_code(200);
  web_server.end_headers();
  if (sensorMagnetTuerWert == 0 && sensorMagnetFensterWert == 0) { // Wenn Tür und Fenster gleichzeitig geöffnet sind, wird ein HTML Code erzeugt, der im Browser eine Grafik mit "Fenster vergessen" anzeigt
    web_server << F("<img width='450' height='450' src='Fenster.png'>") << F("\n");}
  return true;
}

// Sensorhandler für Kühlschrank
boolean sensor_Licht_handler(TinyWebServer& web_server) {
  web_server.send_error_code(200);
  web_server.end_headers();
  if (sensorLichtWert > 800 && sensorMagnetTuerWert == 0) { // Wenn der Lichtsensor genug Licht bekommt, weiß Arduino, dass der Kühlschrank offen ist
    web_server << F("<img width='450' height='450' src='Fridge.png'>") << F("\n");}
  return true;
}
 // Sensorhandler für Herd
boolean sensor_Herd_handler(TinyWebServer& web_server) {
  web_server.send_error_code(200);
  web_server.end_headers();
  // LED-Wechsel
  if (!firstLed && sensorMagnetTuerWert == 0) { // für den Herd wird nur die LED und nicht der Taster überprüft, d.h. wenn die LED leuchtet, weiß Arduino, dass der Herd an ist 
    web_server << F("<img width='450' height='450' src='Herd.png'>") << F("\n");}
  return true;
}


void send_file_name(TinyWebServer& web_server, const char* filename) {
  if (!filename) {
    web_server.send_error_code(404);
    web_server << F("Could not parse URL");
  } else {
    TinyWebServer::MimeType mime_type
      = TinyWebServer::get_mime_type_from_filename(filename);
    web_server.send_error_code(200);
    if (file.open(&root, filename, O_READ)) {
	  web_server.send_content_type(mime_type);
	  web_server.end_headers();
      Serial << F("Read file "); Serial.println(filename);
      web_server.send_file(file);
      file.close();
    } else {
	  web_server.send_content_type("text/plain");
	  web_server.end_headers();
	  Serial << F("Could not find file: "); Serial.println(filename);
      web_server << F("Could not find file: ") << filename << F("\n");
    }
  }
}

// webroot mit der index.htm verknüpfen
boolean index_handler(TinyWebServer& web_server) {
  send_file_name(web_server, "index.htm");
  return true;
}

boolean has_ip_address = false;
TinyWebServer web = TinyWebServer(handlers, NULL);

const char* ip_to_str(const uint8_t* ipAddr)
{
  static char buf[16];
  sprintf(buf, "%d.%d.%d.%d\0", ipAddr[0], ipAddr[1], ipAddr[2], ipAddr[3]);
  return buf;
}

void setup() {
  Serial.begin(9600);
  
  Serial << F("Free RAM: ") << FreeRam() << "\n";
  
  pinMode(10, OUTPUT); // set the SS pin as an output (necessary!)
  digitalWrite(10, HIGH); // but turn off the W5100 chip!
  // initialize the SD card
  
  Serial << F("Setting up SD card...\n");

  if (!card.init(SPI_FULL_SPEED, 4)) {
    Serial << F("card failed\n");
    has_filesystem = false;
  }
  // initialize a FAT volume
  if (!volume.init(&card)) {
    Serial << F("vol.init failed!\n");
    has_filesystem = false;
  }
  if (!root.openRoot(&volume)) {
    Serial << F("openRoot failed");
    has_filesystem = false;
  }

  Serial << F("Setting up the Ethernet card...\n");
  Ethernet.begin(mac, ip);

  // Start the web server.
  Serial << F("Web server starting...\n");
  web.begin();
  
  Serial << F("Free RAM: ") << FreeRam() << F("\n");

  Serial << F("Ready to accept HTTP requests.\n\n");
  
  pinMode(8, OUTPUT);
  digitalWrite(8, HIGH);
  
  pinMode(7, INPUT);
  //digitalWrite(7, LOW);
  
}


void loop() {
  web.process();
 
 
  // LED-Wechsel 
  int val = digitalRead(7);
  if (status != val) { // Änderung am Schalter?
    if (val) { // gedrückt
	digitalWrite(8, firstLed); // Die eine leuchtet
	
	firstLed = !firstLed; // ...welche das ist wird abgewechselt.
    }
    status = val; // Merken
    //delay (1000); // Debounce
  }

  
  // den Sensor nur alle 1000ms auslesen
  if (millis() - sensorMagnetTuerZuletztGelesen > 1000) {
    sensorMagnetTuerAuslesen();
    
    sensorMagnetTuerZuletztGelesen = millis();
  }  
  // den Sensor nur alle 1000ms auslesen
  if (millis() - sensorMagnetFensterZuletztGelesen > 1000) {
    sensorMagnetFensterAuslesen();
    
    sensorMagnetFensterZuletztGelesen = millis();
  }  
  
  // den Sensor nur alle 1000ms auslesen
  if (millis() - sensorLichtZuletztGelesen > 1000) {
    sensorLichtAuslesen();
    
    sensorLichtZuletztGelesen = millis();
  } 
  
}
